/*

  fpgaio.c - map FPGA I/O to kernel memory

  Author: Santeri Paavolainen <santtu@ssh.com>

  Copyright 2002 SSH Communications Security Corp,
  All Rights Reserved. http://www.ssh.com

*/

#include <linux/module.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/init.h>

#include <asm/io.h>
#include <asm/immap_8260.h>
#include <asm/cpm_8260.h>

#include <asm/uaccess.h>
#include <asm/mpc8260.h>
#include <asm/io.h>
#include <asm/immap_8260.h>
#include <asm/cpm_8260.h>

#include "fpgaio.h"

void * sipofpgaio_ptr = NULL;

void __init sipofpgaio_init(void)
{
	volatile immap_t *immap;
	volatile memctl8260_t *memc;
	int i;

	sipofpgaio_ptr = ioremap(FPGA_IO_BASE_ADDR, FPGA_IO_SIZE);

	if (!sipofpgaio_ptr) {
		printk(KERN_ERR
		       "SIPO FPGA IO: "
		       "Failed to ioremap 0x%x (size 0x%x) for FPGA I/O "
		       "access.\n",
		       FPGA_IO_BASE_ADDR, FPGA_IO_SIZE);
		return;
	}

	/* Setup 8260 Memory controller to map FPGA io space */
	immap = (immap_t *)IMAP_ADDR;
	memc = &immap->im_memctl;
	memc->memc_br5 = FPGA_IO_BASE_ADDR | BR_PS64 | BR_MS_UPMA | BR_VALID;
	memc->memc_or5 = ~(FPGA_IO_SIZE - 1);

	/* configure machine mode register for upm program load
	 * GPL4 must not be configured as upm_wait when program is loaded */
	memc->memc_mamr = MMR_B60X | MMR_WRITE;

	/* load upm program */
	for (i = 0;  i < UPM_ARRAYSIZE; i++) {
		/* load data into upm memory data register */
		memc->memc_mdr = upm_array[i];
		mb();

		/* access to memory bank performs upm program write */
		*(uint *)sipofpgaio_ptr = upm_array[i];
		mb();
	}

	/* configure machine mode register for normal operation */
	memc->memc_mamr = MMR_B60X | MMR_NORMAL | MMR_DS1 | MMR_G4DI | MMR_MUX;

	printk("SIPO FPGA IO: "
	       "Mapped 0x%x (0x%x bytes) for FPGA I/O to address %p and loaded"
	       " UPM access program (%d bytes).\n",
	       FPGA_IO_BASE_ADDR, FPGA_IO_SIZE, sipofpgaio_ptr, UPM_ARRAYSIZE);
}

#ifdef MODULE
int init_module(void)
{
	sipofpgaio_init();
	return 0;
}

void cleanup_module(void)
{
	if (sipofpgaio_ptr) {
		iounmap(sipofpgaio_ptr);
		sipofpgaio_ptr = NULL;
	}
}
#endif
