#define FPGA_IO_BASE_ADDR       0xf2000000
#define FPGA_IO_SIZE    0x00010000

/* XXX:

  These are temporary addresses, since this is just a testbench for
  the UART code in a system where addr<0:4> lines are tied to the
  smart card directly. Thus we have to use addr<5:6> for the real
  address selection instead.

*/

#define FPGA_IO_CRYPTO  0
#define FPGA_IO_UART_FIFO 8
#define FPGA_IO_UART_CONTROL 16

extern void * sipofpgaio_ptr;


/* These definitions set signals active for all four clock phases
 * setting individual clock phases is not covered here
 */
#define UPM_CS_0 0x00000000
#define UPM_CS_1 0xF0000000
#define UPM_BS_0 0x00000000
#define UPM_BS_1 0x0F000000
#define UPM_G0_0 0x00A00000
#define UPM_G0_1 0x00F00000
#define UPM_G1_0 0x00000000
#define UPM_G1_1 0x000C0000
#define UPM_G2_0 0x00000000
#define UPM_G2_1 0x00030000
#define UPM_G3_0 0x00000000
#define UPM_G3_1 0x0000C000
#define UPM_G3HL 0x00008000
#define UPM_G4_0 0x00000000
#define UPM_G4_1 0x00003000
#define UPM_G5_0 0x00000000
#define UPM_G5_1 0x00000C00
#define UPM_WAEN 0x00001000
#define UPM_WADI 0x00000000
#define UPM_REP1 0x00000000
#define UPM_REP2 0x00000100
#define UPM_REP3 0x00000200
#define UPM_REP4 0x00000300
#define UPM_LOOP 0x00000080
#define UPM_EXEN 0x00000040
#define UPM_ACOL 0x00000000
#define UPM_AROW 0x00000020
#define UPM_AMAR 0x00000030
#define UPM_NEXT 0x00000008
#define UPM_UTA  0x00000004
#define UPM_TODT 0x00000002
#define UPM_LAST 0x00000001

/* read single beat (RSS) offset 0x00  */
/* read burst (RBS) offset 0x08        */
/* write single beat (WSS) offset 0x18 */
/* write burst (WBS) offset 0x20       */
/* refresh timer (PTS) offset 0x30     */
/* exception condition (EXS) 0x3C      */

#define UPM_ARRAYSIZE 64
static const unsigned long upm_array[UPM_ARRAYSIZE] = {
  /* RSS */
  (UPM_CS_0 | UPM_BS_1 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP2 | UPM_AROW),
  (UPM_CS_1 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA | UPM_LAST),
  0, 0, 0, 0, 0, 0,
  /* RBS */
  (UPM_CS_0 | UPM_BS_1 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP2 | UPM_AROW),
  (UPM_CS_1 | UPM_BS_1 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW),
  (UPM_CS_1 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA),
  (UPM_CS_1 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA),
  (UPM_CS_1 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA),
  (UPM_CS_1 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA | UPM_LAST),
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  /* WSS */
  (UPM_CS_0 | UPM_BS_1 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP2 | UPM_AROW),
  (UPM_CS_1 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA | UPM_LAST),
  0, 0, 0, 0, 0, 0,
  /* WBS */
  (UPM_CS_0 | UPM_BS_1 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW),
  (UPM_CS_0 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA),
  (UPM_CS_1 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA),
  (UPM_CS_1 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA),
  (UPM_CS_1 | UPM_BS_0 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_AROW | UPM_UTA | UPM_LAST),
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  /* PTS */
  (UPM_CS_1 | UPM_BS_1 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_LAST),
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  /* EXS */
  (UPM_CS_1 | UPM_BS_1 | UPM_G0_1 | UPM_G1_1 | UPM_G2_1 |
   UPM_G3_1 | UPM_WADI | UPM_G5_1 | UPM_REP1 | UPM_LAST),
  0, 0, 0
};

#define MMR_B60X   0x00000000
#define MMR_BLOCAL 0x80000000
#define MMR_RFEN   0x40000000
#define MMR_NORMAL 0x00000000
#define MMR_WRITE  0x10000000
#define MMR_READ   0x20000000
#define MMR_RUN    0x30000000
#define MMR_DS1    0x00000000
#define MMR_DS2    0x00400000
#define MMR_DS3    0x00800000
#define MMR_DS4    0x00C00000
#define MMR_G4DI   0x00040000
#define MMR_MUX    0x04000000

#define BR_AMASK   0xFFFF8000
#define BR_PS8     0x00000800
#define BR_PS16    0x00001000
#define BR_PS32    0x00001800
#define BR_PS64    0x00000000
#define BR_MS_UPMA 0x00000080
#define BR_VALID   0x00000001
