/*
 *      Xilinx rbt ASCII bitstream to binary bitstream converter
 *
 *      Copyright 2000 SSH Communications Security Corp, All Rights Reserved.
 *                              http://www.ssh.com
 *
 *      23.9.2000 Arto Vuori <avuori@ssh.com>
 *              Initial version
 *
 *      Reads .rbt file from stdin and writes .bin to stdout.
 *      Binary bitstream format is:
 *
 *      MAGIC_CODE      0xf00ba37e
 *      lenght          Big Endian, number of bits
 *      bits            bitstream, msb first
 *
 *
 */

#include <stdio.h>
#include <string.h>

int main(int argc, char **argv)
{
	char s[1024], arch[1024], name[1024], part[1024], date[1024];
	unsigned int bits, i, a, b, c;

	if (!fgets(s, 1024, stdin)) {
		fprintf(stderr, "Cannot read stdin\n");
		exit(1);
	}

	if (strncmp(s,"Xilinx ASCII Bitstream", 22)) {
		fprintf(stderr, "\"Xilinx ASCII Bitstream\" Expected\n"
			"  got %s\n", s);
		exit(1);
	}

	/* "Created by bitstream D.22" */
	if (!fgets(s, 1024, stdin)) {
		fprintf(stderr, "ERROR\n");
		exit(1);
	}

	if (!fscanf(stdin, "Design name:    %s\n",name) ||
	    !fscanf(stdin, "Architecture:   %s\n", arch) ||
	    !fscanf(stdin, "Part:           %s\n", part) ||
	    !fgets(date, 1024, stdin) ||
	    !fscanf(stdin, "Bits:           %d\n", &bits)) {
		fprintf(stderr, "Invalid input\n");
		exit(1);
	}

	/* Magic */
	fputc(0xf0, stdout);
	fputc(0x0b, stdout);
	fputc(0xa3, stdout);
	fputc(0x7e, stdout);

	/* Length */
	fputc((bits >> 24) & 0xff, stdout);
	fputc((bits >> 16) & 0xff, stdout);
	fputc((bits >> 8) & 0xff, stdout);
	fputc((bits >> 0) & 0xff, stdout);

	/* Bitstream */
	for (i = 0, a = 0; i < bits; ) {
		if ((c = fgetc(stdin)) == EOF) {
			fprintf(stderr, "input too short\n");
			exit(1);
		}

		if (c == '1')
			a |= 1;
		else if (c == '0')
			a &= ~1;
		else
			continue;

		if ((i & 7) == 7) {
			fputc((char) a, stdout);
			a = 0;
		} else
			a <<= 1;

		i++;
	}
	a <<= (7 - (i & 7));
	fputc((char) a, stdout);
}
