/* Routines to switch on and off leds in the SIPO box
 * Copyright 2000 SSH Communications Security Corp
 * Marko Teiste <mte@ssh.com>
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/ptrace.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/spinlock.h>

#include <asm/immap_8260.h>
#include <asm/pgtable.h>
#include <asm/mpc8260.h>
#include <asm/irq.h>
#include <asm/bitops.h>
#include <asm/uaccess.h>
#include <asm/cpm_8260.h>
#include <asm/sipo.h>

static unsigned int led_mask[] = {
	SIPO_LED_BLUE0,
	SIPO_LED_BLUE1,
	SIPO_LED_BLUE2,
	SIPO_LED_BLUE3
};

void sipo_blue_led_on(int led)
{
	volatile immap_t *immap;
	volatile iop8260_t *io;

	immap = (immap_t *)IMAP_ADDR;
	io = &immap->im_ioport;

	io->iop_pdata &= ~(led_mask[led]);
}

void sipo_blue_led_off(int led)
{
	volatile immap_t *immap;
	volatile iop8260_t *io;

	immap = (immap_t *)IMAP_ADDR;
	io = &immap->im_ioport;

	io->iop_pdata |= led_mask[led];
}

void sipo_blue_led_flip(int led)
{
	volatile immap_t *immap;
	volatile iop8260_t *io;

	immap = (immap_t *)IMAP_ADDR;
	io = &immap->im_ioport;

	if (io->iop_pdata & led_mask[led])
		io->iop_pdata &= ~(led_mask[led]);
	else
		io->iop_pdata |= led_mask[led];
}

void sipo_power_led_set(int on)
{
	volatile immap_t *immap;
	volatile iop8260_t *io;

	immap = (immap_t *)IMAP_ADDR;
	io = &immap->im_ioport;

	if (on)
		io->iop_pdatc &= ~SIPO_LED_POWER;
	else
		io->iop_pdatc |= SIPO_LED_POWER;
}

void sipo_status_led_set(int color)
{
	volatile immap_t *immap;
	volatile iop8260_t *io;

	immap = (immap_t *)IMAP_ADDR;
	io = &immap->im_ioport;

	io->iop_pdatc |= SIPO_LED_STATUS_WEIRD;

	switch(color) {
	case 1:         /* Red only */
		io->iop_pdatc &= ~SIPO_LED_STATUS_RED;
		break;
	case 2:         /* Green only */
		io->iop_pdatc &= ~SIPO_LED_STATUS_GREEN;
		break;
	case 3:         /* Red and Green */
		io->iop_pdatc &= ~SIPO_LED_STATUS_WEIRD;
	}
}
