/*
 *      SipO WDT 0.1: Watchdog timer
 *
 *      Copyright 2001 SSH Communications Security Corp, All Rights Reserved.
 *                              http://www.ssh.com
 *
 *      5.2.2001 Arto Vuori <avuori@ssh.com>
 *              Initial version
 */
#include <linux/module.h>
#include <linux/config.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/reboot.h>
#include <linux/smp_lock.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <asm/delay.h>
#include <asm/sipo.h>

#include <asm/immap_8260.h>
#include <asm/cpm_8260.h>
#include <asm/machdep.h>

/* Give 120 seconds coundown at boot */
static unsigned int wdt_countdown = HZ * 120;
/* Give 45 seconds time for the user space watchdog to ping kernel watchdog */
#define WDT_SW_PING_TIMEOUT (HZ * 45)
static unsigned int wdt_enabled=1;

void sipo_wdt_enable() {
	wdt_enabled = 1;
}

void sipo_wdt_disable() {
	wdt_enabled = 0;
}

void sipo_wdt_ping(void)
{
	volatile immap_t *immap = (immap_t *) IMAP_ADDR;
	mb();
	immap->im_siu_conf.sc_swsr = 0x556c;
	mb();
	immap->im_siu_conf.sc_swsr = 0xaa39;
	mb();
}

void sipo_heartbeat(void)
{

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,19))
        heartbeat_count(smp_processor_id()) = 0;
#else
	ppc_md.heartbeat_count = 0;
#endif

	/* Check & decrement counter only if WDT is enabled */
	if (wdt_enabled) {
		/* Do not ping WDT, if timer has expired */
		if (!wdt_countdown)
			return;
		else
			wdt_countdown--;
	}

	sipo_wdt_ping();

	if (!wdt_countdown)
		printk("SipO WDT: Watchdog timer has expired\n");
}

static int wdt_open(struct inode *inode, struct file *file)
{
	return 0;
}

static int wdt_release(struct inode *inode, struct file *file)
{
	return 0;
}

static ssize_t wdt_write(struct file *file, const char *data, size_t len,
			 loff_t *ppos)
{
	char buffer[16];

	if (strncpy_from_user(buffer, data, (len < 16) ? len : 16) < 0)
		return -EFAULT;
	if (len >= strlen("enable") &&
	    !strncmp(buffer, "enable", strlen("enable")))
		sipo_wdt_enable();
	if (len >= strlen("disable") &&
	    !strncmp(buffer, "disable", strlen("disable")))
		sipo_wdt_disable();

	wdt_countdown = WDT_SW_PING_TIMEOUT;

	return len;
}

static struct file_operations wdt_fops=
{
	owner:          THIS_MODULE,
	write:          wdt_write,
	open:           wdt_open,
	release:        wdt_release,
};

static struct miscdevice wdt_miscdev=
{
	SIPOWDT_MINOR,
	"sipowdt",
	&wdt_fops
};

void __init sipowdt_init(void)
{
	misc_register(&wdt_miscdev);
	printk("SipO WDT: 0.1a\n");
}
