#include <linux/config.h>
#include <linux/types.h>
#include "nonstdio.h"
#include "zlib.h"
#include <asm/processor.h>
#include <asm/page.h>
#include <asm/sipo.h>
#include <asm/immap_8260.h>
#include <stdarg.h>

#ifdef CONFIG_SIPO_WDT
void sipo_wdt_ping() 
{
        volatile        immap_t         *immap=(immap_t *)IMAP_ADDR;

        mb();
        immap->im_siu_conf.sc_swsr = 0x556c;
        mb();
        immap->im_siu_conf.sc_swsr = 0xaa39;
        mb();
}
#endif

char *avail_ram;
char *end_avail;

char    cmd_buf[256];
char    *cmd_line = cmd_buf;

char    *root_string = "root=/dev/nfs rw";
char    *ramroot_string = "root=/dev/ram";
char    *romroot_string = "root=/dev/rom";
char    *romdevnum = "0123456789abcdef";

void puts(const char *);
void putc(const char c);
void puthex(unsigned long val);
void _bcopy(char *src, char *dst, int len);
void * memcpy(void * __dest, __const void * __src, int __n);
void gunzip(void *, int, unsigned char *, int *);

void pause()
{
        puts("pause\n");
}

void exit()
{
        puts("exit\n");
        while(1); 
}

tstc(void)
{
#ifdef DISABLE_SERIAL
    return 0;
#else
        return (serial_tstc());
#endif
}

getc(void)
{
        while (1) {
                if (serial_tstc()) return (serial_getc());
        }
}

void 
putc(const char c)
{
        serial_putchar(c);
}

void puts(const char *s)
{
#ifndef DISABLE_SERIAL
        char c;

        while ( ( c = *s++ ) != '\0' ) {
                serial_putchar(c);
                if ( c == '\n' )
                        serial_putchar('\r');
        }
#endif
}

#ifndef DISABLE_SERIAL
extern int vsprintf(char *buf, const char *fmt, va_list args);
static char sprint_buf[1024];

void
printk(char *fmt, ...)
{
        va_list args;
        int n;

        va_start(args, fmt);
        n = vsprintf(sprint_buf, fmt, args);
        va_end(args);
        puts(sprint_buf);
}

int
printf(char *fmt, ...)
{
        va_list args;
        int n;

        va_start(args, fmt);
        n = vsprintf(sprint_buf, fmt, args);
        va_end(args);
        puts(sprint_buf);
        return n;
}
#endif

void puthex(unsigned long val)
{
        unsigned char buf[10];
        int i;
        for (i = 7;  i >= 0;  i--)
        {
                buf[i] = "0123456789ABCDEF"[val & 0x0F];
                val >>= 4;
        }
        buf[8] = '\0';
        puts(buf);
}


void error(char *x)
{
        puts("\n\n");
        puts(x);
        puts("\n\n -- System halted");

        while(1);       /* Halt */
}

void *zalloc(void *x, unsigned items, unsigned size)
{
        void *p = avail_ram;
        
        size *= items;
        size = (size + 7) & -8;
        avail_ram += size;
        if (avail_ram > end_avail) {
                puts("oops... out of memory\n");
                pause();
        }
        return p;
}

void zfree(void *x, void *addr, unsigned nb)
{
}

#define HEAD_CRC        2
#define EXTRA_FIELD     4
#define ORIG_NAME       8
#define COMMENT         0x10
#define RESERVED        0xe0

#define DEFLATED        8


void gunzip(void *dst, int dstlen, unsigned char *src, int *lenp)
{
        z_stream s;
        int r, i, flags;
        
        /* skip header */
        i = 10;
        flags = src[3];
        if (src[2] != DEFLATED || (flags & RESERVED) != 0) {
                puts("bad gzipped data\n");
                exit();
        }
        if ((flags & EXTRA_FIELD) != 0)
                i = 12 + src[10] + (src[11] << 8);
        if ((flags & ORIG_NAME) != 0)
                while (src[i++] != 0)
                        ;
        if ((flags & COMMENT) != 0)
                while (src[i++] != 0)
                        ;
        if ((flags & HEAD_CRC) != 0)
                i += 2;
        if (i >= *lenp) {
                puts("gunzip: ran out of data in header\n");
                exit();
        }
        
        s.zalloc = zalloc;
        s.zfree = zfree;
        r = inflateInit2(&s, -MAX_WBITS);
        if (r != Z_OK) {
                puts("inflateInit2 returned %d\n");
                exit();
        }
        s.next_in = src + i;
        s.avail_in = *lenp - i;
        s.next_out = dst;
        s.avail_out = dstlen;
        r = inflate(&s, Z_FINISH);
        if (r != Z_OK && r != Z_STREAM_END) {
                puts("inflate returned %d\n");
                exit();
        }
        *lenp = s.next_out - (unsigned char *) dst;
        inflateEnd(&s);
}

extern void udelay(int);

unsigned char sanity[0x2000];

extern char image_data[];
extern int image_len;
extern char initrd_data[];
extern int initrd_len;

void
start(bd_t *bdt, char *cmdt)
{
        int timer;
        unsigned int len;
        char ch;
        void *dst;
        unsigned char *im;
        unsigned int i;
        char    *rp;
        char    *cmd_line;
        bd_t    *__bd;

        avail_ram = (void *)bdt->bi_memsize-0x100000;
        end_avail = (void *)bdt->bi_memsize-0x10000;

        cmd_line = cmdt;
        __bd = bdt;

        /* If the command line is not filled in, we will automatically
         * create the default boot.
         */
        if (cmd_line[0] == 0) 
        {
                /* An initrd on these boards means we booted from Flash
                 * ROM and want to use the ramdisk as the root file system.
                 * If no initrd is given, try to find rootdisk from flash partition table.
                 * Otherwise, we perform a diskless NFS boot.
                 */
                rp = root_string;
                if (initrd_len>0)
                        rp = ramroot_string;
                else {
                        for(i=0;i<pt_size && i<strlen(romdevnum);i++)
                                if(__bd->pt[i].type==pt_rootdisk) {
                                        rp=romroot_string;                                      
                                        break;
                                }
                }
                strcpy(cmd_line,rp);

                if(rp==romroot_string) {
                    cmd_line[strlen(rp)]=romdevnum[i];
                    cmd_line[strlen(rp)+1]=0;
                }
        }
        
#ifndef DISABLE_SERIAL
        serial_init();
#endif

        im = image_data;
        len = image_len;
        dst = (void *)0x0;
        if (im[0] == 0x1f && im[1] == 0x8b) {
                gunzip(dst, 0x200000, im, &len);
        } else {
                memmove(dst, im, len);
        }

        flush_cache(dst, len);

#ifdef CONFIG_SIPO_WDT
        /* After decompressing kernel, but before starting it, ping watchdog once */
        sipo_wdt_ping();
#endif

        (*(void (*)())0x0)((int)__bd,
                           (initrd_len>0) ? initrd_data : 0,
                           (initrd_len>0) ? (int)initrd_data + initrd_len : 0,
                           cmd_line,
                           cmd_line + strlen(cmd_line));

        pause();
}
