#include <asm/ptrace.h>
#include <linux/types.h>
#include "82xx_immap.h"

#define IMAP ((volatile immap_t *)0xf0000000)

#define BYTE char
#define UBYTE volatile u_char
#define UWORD volatile uint
#define UHWORD volatile ushort
#define VUBYTE volatile u_char
#define VUWORD volatile uint
#define VUHWORD volatile ushort

#define SMC_REG 0          /* 0 for SMC1, 1 for SMC2 */

/* Single buffer component */
typedef struct BufferPool
{
   UBYTE  RxBuffer;
   UBYTE  TxBuffer;
} LB;


/* Buffer Descriptor Format */
typedef struct BufferDescriptor 
{
   UHWORD bd_cstatus;     /* control and status */
   UHWORD bd_length;      /* transfer length */
   UBYTE  *bd_addr;       /* buffer address */
} BD;
 

/*-------------------------------*/
/* Buffer Descriptor Ring format */
/*-------------------------------*/

typedef struct BufferDescRings 
{
    BD RxBD;    /* Rx BD ring */
    BD TxBD;    /* Tx BD ring */
} BDRINGS;

void  serial_init(void);
void  serial_putchar(UBYTE);
UBYTE serial_getc(void);
UBYTE serial_tstc(void);
static void  InitBDs(void);

static volatile BDRINGS *RxTxBD;  /* buffer descriptors base pointer */
static LB *SMCBuffers; 
static t_Smc_Pram *SMC; /* SMC parameter RAM */

/* Set a baud rate generator.  This needs lots of work.  There are
 * four BRGs, any of which can be wired to any channel.
 * The internal baud rate clock is the system clock divided by 16.
 * This assumes the baudrate is 16x oversampled by the uart.
 */
#define BRG_INT_CLK     (133000000/2)
#define BRG_UART_CLK    (BRG_INT_CLK/16)

/* Baud rate generators.
*/
#define CPM_BRG_RST             ((uint)0x00020000)
#define CPM_BRG_EN              ((uint)0x00010000)
#define CPM_BRG_EXTC_INT        ((uint)0x00000000)
#define CPM_BRG_EXTC_CLK2       ((uint)0x00004000)
#define CPM_BRG_EXTC_CLK6       ((uint)0x00008000)
#define CPM_BRG_ATB             ((uint)0x00002000)
#define CPM_BRG_CD_MASK         ((uint)0x00001ffe)
#define CPM_BRG_DIV16           ((uint)0x00000001)


void
m8xx_cpm_setbrg(uint brg, uint rate)
{
        volatile uint   *bp;

        /* This is good enough to get SMCs running.....
        */
        if(brg<5) 
          bp = (&IMAP->brgs_brgc1)+brg-1;
        else
          bp = (&IMAP->brgs_brgc5)+brg-5;

        *bp = ((BRG_UART_CLK / rate) << 1) | CPM_BRG_EN;
}

void serial_init(void)
{
  uint          mem_addr, dp_addr;

   /* Allocate SMC parameter ram */
   mem_addr = 0x80; /* m8xx_cpm_dpalloc(0x80); */
#if SMC_REG
   *(volatile unsigned short *)(IMAP->pram2.standard.smc2) = mem_addr;
#else
   *(volatile unsigned short *)(IMAP->pram2.standard.smc1) = mem_addr;
#endif

   SMC=(t_Smc_Pram *)&IMAP->pram1[mem_addr];

   /* Allocate some PRAM for BD's & buffers */
   dp_addr = 0x800; /* m8xx_cpm_dpalloc(0x80); */

   /* Get pointer to BD area on DP RAM */
   RxTxBD = (BDRINGS *)&IMAP->pram1[dp_addr];

   SMC->rbase = dp_addr;
   SMC->tbase = dp_addr+sizeof(BD);

#if 1
   /* Establish the buffer pool in Dual Port RAM. */
   SMCBuffers = (LB *)&IMAP->pram1[dp_addr+2*sizeof(BD)];
#else
   SMCBuffers = 0xfe0000;
#endif

   /* 
    * Allow SMC TX, RX out 
    */

#if SMC_REG
   /* SMC2 */
   IMAP->io_regs[0].ppar |= 0x00C00000;     
   IMAP->io_regs[0].pdir |= 0x00400000;
   IMAP->io_regs[0].pdir &=~0x00800000;
   IMAP->io_regs[0].psor &=~0x00c00000;
#else 
   /* SMC1 */
   IMAP->io_regs[3].ppar |= 0x00C00000;     
   IMAP->io_regs[3].pdir |= 0x00400000;
   IMAP->io_regs[3].pdir &= 0xFF7FFFFF;
   IMAP->io_regs[3].psor &= 0xFF3FFFFF;
#endif
   

   /* 
    * Set Baudrate generator
    * use BRG7 for SMC1 and BRG8 for SMC2
    */

   IMAP->clocks_sccr = 0;       /* DIV 4 BRG */

#if SMC_REG
   m8xx_cpm_setbrg(2, 9600);
   IMAP->cpm_mux_cmxsmr &= ~0x0f;
   IMAP->cpm_mux_cmxsmr |=  0x00;
#else
   m8xx_cpm_setbrg(1, 9600);
   IMAP->cpm_mux_cmxsmr &= ~0xf0;
   IMAP->cpm_mux_cmxsmr |=  0x00;
#endif

   /* Initialize Rx and Tx Params for SMC: */
   /* Spin until cpcr flag is cleared      */

#if SMC_REG
   for(IMAP->cpm_cpcr = 0x21210000; IMAP->cpm_cpcr & 0x00010000;) ;
#else
   for(IMAP->cpm_cpcr = 0x1d010000; IMAP->cpm_cpcr & 0x00010000;) ;
#endif
   
   /* Set RFCR,TFCR -- Rx,Tx Function Code */
   /* Normal Operation and Motorola byte   */
   /* ordering                             */

   SMC->rfcr = 0x10;
   SMC->tfcr = 0x10;

   SMC->mrblr = 1;
   SMC->maxidl = 0;  
   SMC->brkec = 0;    
   SMC->brkcr = 1;    

   InitBDs();

   IMAP->smc_regs[SMC_REG].smce = 0xFF;  /* Clear any pending events */
   IMAP->smc_regs[SMC_REG].smcm = 0;     

#if 0 
   IMAP->cpmi_cicr = 0;           /* Disable all CPM interrups */
   IMAP->cpmi_cipr = 0xFFFFFFFF;  /* Clear all pending interrupt events */
   IMAP->cpmi_cimr = 0;           /* Mask all event interrupts */
#endif

   IMAP->smc_regs[SMC_REG].smcmr = 0x4820;

#if 1 /* Normal operation */
   IMAP->smc_regs[SMC_REG].smcmr = 0x4823;
#else /* Loopback */
   IMAP->smc_regs[SMC_REG].smcmr = 0x4827;
#endif


}



static void InitBDs(void)
{
   /* Setup Receiver Buffer Descriptors */
   
   RxTxBD->RxBD.bd_cstatus = 0xA000;            /* Enable, Last BD */
   RxTxBD->RxBD.bd_length = 1;
   RxTxBD->RxBD.bd_addr = &(SMCBuffers->RxBuffer);

   /* Setup Transmitter Buffer Descriptors */

   RxTxBD->TxBD.bd_cstatus = 0x2000;      /* Buffer not yet ready; Last BD */
   RxTxBD->TxBD.bd_length = 1;
   RxTxBD->TxBD.bd_addr = &(SMCBuffers->TxBuffer);


} 


void serial_putchar(UBYTE ch)
{
  /* Loop until transmission completed */
   while (RxTxBD->TxBD.bd_cstatus  &  0x8000);    
   
   *(RxTxBD->TxBD.bd_addr) = ch;                  
   RxTxBD->TxBD.bd_length = 1;

   /* Set Ready bit */
   RxTxBD->TxBD.bd_cstatus |= 0x8000;              


} 



UBYTE serial_getc(void)
{
  UBYTE ch;
   
  /* Loop if RxBD empty */
   while (RxTxBD->RxBD.bd_cstatus  &  0x8000);      

   /* Receive data */
   ch = *(RxTxBD->RxBD.bd_addr);   
                   
   /* Set Buffer Empty bit */
   RxTxBD->RxBD.bd_cstatus |= 0x8000;   

   return ch;

}


UBYTE serial_tstc(void)
{
      
   if(RxTxBD->RxBD.bd_cstatus & 0x8000) 
       return 0;  /*  character NOT available */
   else
       return 1;  /*  character IS available */
}

