
/* Board information for the SIPO bootloader v0.3 board
 */

#ifndef __CONFIG_SIPO_DEFS_
#define __CONFIG_SIPO_DEFS_

#define _IMAP_ADDR       0xf0000000
#define IMAP_ADDR        ((uint)_IMAP_ADDR)

#define SIPO_MAGIC 0xf00ba37e

#ifndef __ASSEMBLY__

#include <linux/types.h>

typedef unsigned char SHA1Digest[20];

typedef enum {
    pt_unused = 0x00,
    pt_bootloader = 0x01,
    pt_bd = 0x02,
    pt_kernel = 0x03,
    pt_rootdisk = 0x04,
    pt_initrd = 0x05,
    pt_fs = 0x06 /* JFFS */,
    pt_jffs2 = 0x07,
    pt_last = 0x1f,
    pt_size = 0x20,
    pt_magic = SIPO_MAGIC,
    pt_error = 0xffffffff,
} sipo_partition_type;

typedef struct {
    sipo_partition_type type;
    unsigned            offset;
    unsigned            size;                   /* In bytes */
} sipo_flash_partition;

typedef struct {
    /* EST/Linux stuff... */

    unsigned int        bi_memstart;    /* Memory start address */
    unsigned int        bi_memsize;     /* Memory (end) size in bytes */
    unsigned int        bi_memtotal;     /* Memory (end) size in bytes */
    unsigned int        bi_intfreq;     /* Internal Freq, in Hz */
    unsigned int        bi_busfreq;     /* Bus Freq, in MHz */
    unsigned int        bi_cpmfreq;     /* CPM Freq, in MHz */
    unsigned int        bi_brgfreq;     /* BRG Freq, in MHz */
    unsigned int        bi_vco;         /* VCO Out from PLL */
    unsigned int        bi_baudrate;    /* Default console baud rate */
    unsigned int        bi_immr;        /* IMMR when called from boot rom */
    unsigned char       bi_enetaddr[6];

    /* Sipo extensions... */

    unsigned int        flash_base;
    unsigned int        flash_size;
    unsigned int        true_flash_base;
    unsigned int        bd_version;             /* Board version descriptor */
    unsigned int        hw_version;             /* hardware version */
    unsigned int        sw_version;             /* software version */
    unsigned int        boot_state;             /* Boot state copied from NVRAM at initialization */
    unsigned int        enable_netboot;
    unsigned int        enable_debugport;
    unsigned int        enable_passwd;
    unsigned int        enable_wdt;
    SHA1Digest          salt;
    SHA1Digest          passwd;                 /* digest of salt + passwd */
    sipo_flash_partition pt[pt_size];
    char                cmdline[0x100];         /* Default kernel commandline */
    char                reserved[0xcec];        /* sizeof(sipo_board_info) == 0x1000 */
    SHA1Digest          verify;
} bd_t;

/*
 * SIPO nvram information structure
 */

typedef enum {
    nv_none = 0,
    nv_normal = 1,
    nv_prepare = 2,
    nv_test = 3,
    nv_prepare_part2 = 4,
    nv_error=0xffff
} sipo_nv_state;

typedef struct {
    unsigned int        magic;          /* SIPO_MAGIC */
    sipo_nv_state       state;
    char                _res[0x18];
} sipo_nv_info;


/* And then add some of our own
 */

extern void sipo_blue_led_on(int n);
extern void sipo_blue_led_off(int n);
extern void sipo_blue_led_flip(int n);
extern void sipo_power_led_set(int n);
extern void sipo_status_led_set(int n);

extern unsigned int sipo_fcc_led_show;

#define SIPO_ADDR_LEDS          ((uint)0xf001000)
#define SIPO_LED_BLUE0          (0x80000000>>22)
#define SIPO_LED_BLUE1          (0x80000000>>23)
#define SIPO_LED_BLUE2          (0x80000000>>24)
#define SIPO_LED_BLUE3          (0x80000000>>25)
#define SIPO_LED_STATUS_GREEN   (0x80000000>>7)
#define SIPO_LED_STATUS_RED     (0x80000000>>8)
#define SIPO_LED_STATUS_WEIRD   (SIPO_LED_STATUS_RED \
                                |SIPO_LED_STATUS_GREEN)
#define SIPO_LED_POWER          (0x80000000>>6)
#define SIPO_LEDS_PORTA_MASK    (SIPO_LED_BLUE0 \
                                |SIPO_LED_BLUE1 \
                                |SIPO_LED_BLUE2 \
                                |SIPO_LED_BLUE3)
#define SIPO_LEDS_PORTC_MASK    (SIPO_LED_STATUS_WEIRD \
                                |SIPO_LED_POWER)

/* RTC Stuff */
extern long sipo_time_init(void);
extern unsigned long sipo_get_rtc_time(void);
extern int sipo_set_rtc_time(unsigned long);

extern void sipo_init(void);
extern void *sipo_flash_base;
extern uint  sipo_flash_size;

extern void sipoled_power_blink(int flag);

extern void proc_sipo_init(void);
extern struct proc_dir_entry * sipo_proc_root_dir; /* /proc/sipo */

extern int flash_init(void);
extern int flash_read(char * buf, uint ppos, size_t count);
extern int flash_write(char * buf, uint ppos, size_t count);
extern int flash_erase(uint ppos, size_t count);

extern void sipo_wdt_enable(void);
extern void sipo_wdt_disable(void);

#define IOCTL_FLASH_BLOCK_ERASE 6789

#endif /* __ASSEMBLY__ */

#endif  /* __CONFIG_SIPO_DEFS_ */
