/* 
   sipo board descriptor initialization values 
*/

#define __BD_memstart           0
#define __BD_memsize            (0x3f<<20);
#define __BD_memtotal           (0x40<<20);
#define __BD_vco                0xff
#define __BD_baudrate           9600
#define __BD_immr               IMAP_ADDR
#define __BD_enetaddr           {0xde,0xad,0xde,0xad,0xde,0xad}
#define __BD_flash_base         FLASH_START_ADDRESS
#define __BD_bd_version         0x04
#define __BD_sw_version         0x01
#define __BD_boot_state         0x00
#define __BD_enable_wdt         1
#define __BD_enable_netboot     0
#define __BD_enable_debugport   0
#define __BD_cmdline            ""

#define SIPO_SECRET { \
0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, \
0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x14 }
 
#define SIPO_MAGIC 0xf00ba37e

#define FLASH_START_ADDRESS 0xfe000000
#define BD_START_ADDRESS        (FLASH_START_ADDRESS + BD_OFFSET)

/*
 * Hardware version specific information
 */

#if __BD_hw_version == 0x02
#define FLASH_BD_BLOCK         4
#define FLASH_BLOCKSIZE (64 * 1024)
#define FPGA_INIT_PIN        (0x80000000>>23)
#define FPGA_INIT_PORT       2
#define __BD_intfreq            166666666
#define __BD_busfreq            66666666
#define __BD_cpmfreq            133333333
#define __BD_brgfreq            66666666

#elif __BD_hw_version == 0x03
#define FLASH_BD_BLOCK         2
#define FLASH_BLOCKSIZE (128 * 1024)
#define FPGA_INIT_PIN        (0x80000000>>23)
#define FPGA_INIT_PORT       2
#define __BD_intfreq            166666666
#define __BD_busfreq            66666666
#define __BD_cpmfreq            133333333
#define __BD_brgfreq            66666666

#elif __BD_hw_version == 0x04         /* SIPO2 */
#define FLASH_BD_BLOCK         2
#define FLASH_BLOCKSIZE (128 * 1024)
#define FPGA_INIT_PIN        (0x80000000>>11)
#define FPGA_INIT_PORT       3
#define __BD_intfreq            166666666
#define __BD_busfreq            66666666
#define __BD_cpmfreq            133333333
#define __BD_brgfreq            66666666

#elif __BD_hw_version == 0x05        /* 50MHz 8255 */
#define FLASH_BD_BLOCK         2
#define FLASH_BLOCKSIZE (128 * 1024)
#define FPGA_INIT_PIN        (0x80000000>>23)
#define FPGA_INIT_PORT       2
#define __BD_intfreq            200000000
#define __BD_busfreq            50000000
#define __BD_cpmfreq            100000000
#define __BD_brgfreq            50000000

#elif __BD_hw_version == 0x06        /* Same as 0x03, but only two Ethernet
                                      * interfaces */
#define FLASH_BD_BLOCK         2
#define FLASH_BLOCKSIZE (128 * 1024)
#define FPGA_INIT_PIN        (0x80000000>>23)
#define FPGA_INIT_PORT       2
#define __BD_intfreq            166666666
#define __BD_busfreq            66666666
#define __BD_cpmfreq            133333333
#define __BD_brgfreq            66666666

#elif __BD_hw_version == 0x07        /* Same as 0x03 (rack) */
#define FLASH_BD_BLOCK         2
#define FLASH_BLOCKSIZE (128 * 1024)
#define FPGA_INIT_PIN        (0x80000000>>23)
#define FPGA_INIT_PORT       2
#define __BD_intfreq            166666666
#define __BD_busfreq            66666666
#define __BD_cpmfreq            133333333
#define __BD_brgfreq            66666666

#else
#ifndef NATIVE
#error "__BD_hw_version not defined"
#endif
#endif

#define BD_OFFSET               (FLASH_BLOCKSIZE*FLASH_BD_BLOCK - sizeof(sipo_board_info))

typedef unsigned char SHA1Digest[20];

typedef enum {
    pt_unused = 0x00,
    pt_bootloader = 0x01,
    pt_bd = 0x02,
    pt_kernel = 0x03,
    pt_rootdisk = 0x04,
    pt_initrd = 0x05,
    pt_fs = 0x06 /* JFFS */,
    pt_jffs2 = 0x07,
    pt_last = 0x1f,
    pt_size = 0x20,
    pt_magic = SIPO_MAGIC,
    pt_error = 0xffffffff,
} sipo_partition_type;

typedef struct {
    sipo_partition_type type;
    unsigned            offset;
    unsigned            size;                   /* In bytes */    
} sipo_flash_partition;

typedef struct {
    /* EST / Linux basic info.. */
    unsigned int        bi_memstart;    /* Memory start address */
    unsigned int        bi_memsize;     /* Memory (end) size in bytes */
    unsigned int        bi_memtotal;     /* Memory (end) size in bytes */
    unsigned int        bi_intfreq;     /* Internal Freq, in Hz */
    unsigned int        bi_busfreq;     /* Bus Freq, in MHz */
    unsigned int        bi_cpmfreq;     /* CPM Freq, in MHz */
    unsigned int        bi_brgfreq;     /* BRG Freq, in MHz */
    unsigned int        bi_vco;         /* VCO Out from PLL */
    unsigned int        bi_baudrate;    /* Default console baud rate */
    unsigned int        bi_immr;        /* IMMR when called from boot rom */
    unsigned char       bi_enetaddr[6];

    /* Sipo extended info... */
    unsigned int        flash_base;
    unsigned int        flash_size;
    unsigned int        true_flash_base;
    unsigned int        bd_version;             /* Board version descriptor */
    unsigned int        hw_version;             /* hardware version */
    unsigned int        sw_version;             /* software version */
    unsigned int        boot_state;             /* Boot state copied from NVRAM at initialization */
    unsigned int        enable_netboot;        
    unsigned int        enable_debugport;
    unsigned int        enable_passwd;
    unsigned int        enable_wdt;
    SHA1Digest          salt;
    SHA1Digest          passwd;                 /* Hash of passwd */
    sipo_flash_partition pt[pt_size];
    char                cmdline[0x100];         /* Default kernel commandline */
    char                reserved[0xce8];        /* sizeof(sipo_board_info) == 0x1000 */
    SHA1Digest          verify;
} sipo_board_info;

typedef sipo_board_info bd_t;

typedef enum {
    nv_none = 0,
    nv_normal = 1,
    nv_prepare = 2,
    nv_test = 3,
    nv_finalize = 4,
    nv_minimal = 5,
    nv_recovery = 6,
    nv_error=0xffff
} sipo_nv_state;
    
typedef struct {
    unsigned int        magic;          /* SIPO_MAGIC */
    sipo_nv_state       state;
    char                _res[0x18];
} sipo_nv_info;
