/*******************************************************************************

  MPC8260 Configuration
  ---------------------

  This file contains MPC8260 hardware configuration word + important system
  dependant constans required in boot-up initialization.

  Author: Arto Vuori <avuori@ssh.com>

  Copyright (C) 2000 SSH Communications Security Corp, Helsinki, Finland.
  All rights reserved. 

*******************************************************************************/


/* Upper word of Internal Memory Map base address */
#define IMMR_HI	 0xf000

/* Flash controller settings */
#define FLASH_START_ADDRESS 0xfe000000
#define FLASH_OR 0xff0008f6	
#define FLASH_BR 0xfe001001
#define FLASH_BR_BOOT 0x00001001

/*
 *	SDRAM controller
 */

#define SDRAM_OR (0xFC000000+(SDRAM_BPD<<13)+(SDRAM_ROWST<<9)+(SDRAM_NUMR<<6)+(SDRAM_IBID<<4))
#define SDRAM_BR 0x00000041	
#define SDRAM_BR_BOOT 0x10000041
	
/* BPD (Banks Per Device) = 00 = 2 internal banks per device. */
#define SDRAM_BPD 1
/* ROWST (Row Start Address Bit) = 011 = A9 being the row start address bit */
#define SDRAM_ROWST 6
/* NUMR (Number of Row Address Lines) = 010 = 11 row address lines. */
#define SDRAM_NUMR 3
/* IBID (Internal bank interleaving within same device disable) */
#define SDRAM_IBID 1

#define SDRAM_PBI 1
#define SDRAM_OP_NORMAL 0x4000
#define SDRAM_OP_PRECHARGE 0x2800
#define SDRAM_OP_CBRREFR 0x0800
#define SDRAM_OP_MODEWR 0x1800
#define SDRAM_SDAM 3	
#define SDRAM_RFRC 6
#define SDRAM_BSMA 2
#define SDRAM_SDA10 4
#define SDRAM_PRETOACT 1
#define SDRAM_ACTTORW 1
#define SDRAM_LDOTOPRE 2
#define SDRAM_WRC 1
#define SDRAM_EAMUX 1
/* CAS latency */
#define SDRAM_CL 2
#define SD_PSDMR (SDRAM_PBI<<15)+(SDRAM_SDAM<<8)+(SDRAM_BSMA<<5)+(SDRAM_SDA10<<2)+(SDRAM_RFRC>>1)
#define SD_PSDMR_LO ((SDRAM_RFRC & 1)<<15)+(SDRAM_PRETOACT<<12)+(SDRAM_LDOTOPRE<<6)+(SDRAM_WRC<<4)+(SDRAM_EAMUX<<3)+(SDRAM_ACTTORW<<9)+SDRAM_CL

#define SDRAM_BURSTLEN 2
	
/*
 * Led bits
 */

#define SIPO_LED_BLUE0          (0x80000000>>22)
#define SIPO_LED_BLUE1          (0x80000000>>23)
#define SIPO_LED_BLUE2          (0x80000000>>24)
#define SIPO_LED_BLUE3          (0x80000000>>25)
#define SIPO_LED_STATUS_GREEN   (0x80000000>>7)
#define SIPO_LED_STATUS_RED	(0x80000000>>8)
#define SIPO_LED_STATUS_WEIRD	(SIPO_LED_STATUS_RED|SIPO_LED_STATUS_GREEN)
#define SIPO_LED_POWER		(0x80000000>>6)
#define SIPO_LEDS_PORTA_MASK	(SIPO_LED_BLUE0|SIPO_LED_BLUE1|SIPO_LED_BLUE2|SIPO_LED_BLUE3)
#define SIPO_LEDS_PORTC_MASK	(SIPO_LED_STATUS_WEIRD|SIPO_LED_POWER)
	
/*
 *	Hardware configuration word
 */ 
	
/* EARB (External Arbitration) = 0 = Arbitration is internal. */	
#define HWC_EARB	0	

/* EXMC (External MEMC) = 0 = */
#define HWC_EXMC 0
	
/* CDIS (Core DISable) = 0 = The CPU core is active. */
#define HWC_CDIS 0
	
/* EBM (External Bus Mode) = 0 = Single PowerQUICCII bus mode is assumed.*/
#define HWC_EBM	1
	
/* BPS (Boot Port Size) = 11b = 32 bit port size. */
#define HWC_BPS	2

/* CIP (Core Initial Prefix) = 1 =  Reset vector is at 0 */
#define HWC_CIP 1
	
/* ISPS (Internal Space Port Size) = 0 = 64bit */
#define HWC_ISPS 0		

/* L2CPC (L2 Cache Pins Configuration) = 10b */
#define HWC_L2CPC 2

/*  DPPC (Data Parity Pin Configuration) = 11b */
#define HWC_DPPC 0

/* ISB (initial Internal Space Base select) = 100b = 0xf0000000 */
#define HWC_ISB 4	

/* BMS (Boot Memory Space) = 0 = 0xFE000000 */
#define HWC_BMS 1

/* BBD (Bus Busy Disable) = 0 =  */
#define HWC_BBD 0
	
/* MMR (Mask Masters Requests) = 00b */
#define HWC_MMR 0
	
/* LBPC (Local Bus Pin Configuration) = 00b */
#define HWC_LBPC 0	

/* APPC (Address Parity Pin Configuration) = 10b */
#define HWC_APPC 2
		
/* CS10PC (CS10 Pin Configuration) = 00b */
#define HWC_CS10PC 1
		
/* MODCK_HI (Clock Configuration Bits) = 0101b */
#define HWC_MODCK 5

/* 
 *	Compose HWC. This should be located at the very beginning on object image. (that is address 0) 
 */

 .text
 .align 2
		
 .fill   1,1,(HWC_EARB<<7)|(HWC_EXMC<<6)|(HWC_CDIS<<5)|(HWC_EBM<<4)|(HWC_BPS<<2)|(HWC_CIP<<1)|(HWC_ISPS)
 .fill   7,1,0   
 .fill   1,1,(HWC_L2CPC<<6)|(HWC_DPPC<<4)|(HWC_ISB)
 .fill   7,1,0
 .fill   1,1,(HWC_BMS<<7)|(HWC_BBD<<6)|(HWC_MMR<<4)|(HWC_LBPC<<2)|(HWC_APPC)
 .fill   7,1,0
 .fill   1,1,(HWC_CS10PC<<6)|(HWC_MODCK)
 .fill   231,1,0 
