/*-----------------------------------------------------------------------               
*
* File:  ethernet.h
*
* Description:  
*
*       Constants and Definitions for MPC8260 Fast Ethernet Example.
*
*
* History:
*
* 15 SEP 98    ggh     Initial FE version.
*
* 24 NOV 99    jms     Modified code for PILOT Rev boards.  The code now reads
*                      BCSR2 for the board revision and then chooses the correct
*                      bit positionings for BCSR0 and BCSR1.
*  
*
*-----------------------------------------------------------------------*/


/*----------------------------------------*/
/* Constants and Definitions for Ethernet */
/*----------------------------------------*/

#define ENET_C_PRES    0xFFFFFFFF    /* CRC Preset */
#define ENET_C_MASK    0xDEBB20E3    /* Constant MASK for CRC */
#define ENET_MFLR      1518          /* Ethernet Max Frame Length */
#define ENET_MINFLR    64            /* Ethernet Min Frame Length */
#define ENET_MDMA      1520          /* Max DMA length */
#define ENET_RET_LIM   15            /* Retry Limit Threshold */
#define ENET_PAD       0x8888        /* Pad Characters */
#define ENET_DSR       0xD555        /* DSR value for Ethernet */
#define ENET_PADDR_H   0x0123        /* Physical Address 1 (MSB) */
#define ENET_PADDR     0xdead        /* Physical Address */
#define ENET_PADDR_L   0x4567        /* Physical Address 1 (LSB) */
#define ENET_MRBLR     2044          /* Max. Receive Buffer Length
                                        (arbitrarily a little longer
                                        than expected BD size for this
                                        example) */
 
/*--------------------------------*/
/* Size of buffers in buffer pool */
/*--------------------------------*/

#define BUFFER_SIZE 2048

/*---------------------------------------------------*/
/* Number of Receive and Transmit Buffer Descriptors */
/*---------------------------------------------------*/

#define NUM_RXBDS 4
#define NUM_TXBDS 4

typedef struct BufferDescriptor 

{
   unsigned short bd_cstatus;     /* control and status */
   unsigned short bd_length;      /* transfer length */
   unsigned char  *bd_addr;       /* buffer address */
} bufferdesc;


#define BD_RX_ERROR 0x3f    /* Mask for set of Receive Buffer Errors 
                               including: LG, NO, SH, CR, OV, CL */
                                                              

/*------------------*/
/* Additional Masks */
/*------------------*/

/* GFMR MASKS */

#define GFMR_ENT  0x00000020
#define GFMR_ENR  0x00000010


/* CPM_CPCR MASKS */

#define CPCR_FLG                0x00010000
#define CPCR_FCC1_CH            0x12000000
#define CPCR_FCC2_CH            0x16200000
#define CPCR_FCC3_CH            0x1a400000
#define CPCR_GRSTOP_TX          0x00000005
#define CPCR_INIT_TX_RX_PARAMS  0x00000000
#define CPCR_MCN_FEC            0x00000300
#define READY_TO_RX_CMD         0  /* Ready to receive a command */




