	.text
	.align 2

#include "ppc_asm.tmpl"	
#include "init8260.h" 
#include "config.S"
#include "vect_tbl.S"

/*
  The factory burnt default code for booting a SSH Creon board.
  This code will initialize the CPU with a mini-stack inside the
  8260 chip.
*/
	.globl  _start    
_start:
        mfmsr    r3
        ori      r3,r3,0x1000    /* Set ME flags */
        mtmsr    r3	
	
	/* Keep pointer to IMMR in r4 */
	addis    r4,0,IMMR_HI+1

	bl	init_leds
	
        /* Turn on POWER led at startup. */
	addis	r3, 0, (SIPO_LED_POWER)@h
	ori	r3, r3, (SIPO_LED_POWER)@l
	bl	set_leds
	
	/* Call 82xx_init to finish chip initialization. */
	bl	mpc82xx_init
#if 1
	/* Initialize SDRAM */		
	bl	init_sdram	        
#endif
	/* Set stack to in-chip memory */
	bl	init_ministack

	/* Initialization before SDRAM */
	bl	_init

	
	/* Move flash content to RAM and swap memory mapping */
        bl	copy_code

	/* Set stack in RAM space */
	bl	init_stack

	/* The stack has been established now so interrupts are recoverable. Set 
	   MSR[RI] bit. */	
	mfmsr    r3
	ori      r3,r3,0x8002		/* Set EE & RI flag */
	mtmsr    r3

	bl	main			/* Jump to main() */		
exit:
	b	exit			/* If there is a return from main(), loop forever */


/* Use this as start point when burning flash with debugger */
	.global _flash_start
_flash_start:	
        mfmsr    r3
        ori      r3,r3,0x1000    /* Set ME flags */
        mtmsr    r3	
	
	/* Keep pointer to IMMR in r4 */
	addis    r4,0,IMMR_HI+1

	/* Do some entertainment */
	bl	init_leds
	
	bl init_stack
	
	mfmsr    r3
	ori      r3,r3,0x0002	/* Set RI flag */
	mtmsr    r3

	bl flashmain	
	b  exit

/* Start from here to debug bootloader */	
	.global _debug_start	
_debug_start:	
        mfmsr    r3
        ori      r3,r3,0x1000    /* Set ME flags */
        mtmsr    r3	
	
	/* Keep pointer to IMMR in r4 */
	addis    r4,0,IMMR_HI+1

	bl	init_leds
	
	bl       init_siu

	/* Set stack in RAM space */
	bl	init_stack

	bl	serial_init

	/* The stack has been established now so interrupts are recoverable. Set 
	   MSR[RI] bit. */	
	mfmsr    r3
	ori      r3,r3,0x8002		/* Set EE & RI flag */
	mtmsr    r3

	bl	main			/* Jump to main() */
	b	exit
		
/************************************************************************
*
* Function: mpc82xx_init
*
* Description: This function initializes the rest of the mpc82xx with the 
*              exception of the CPM.
*
************************************************************************/

mpc82xx_init:	
	mfspr    r31,LR

	/* Keep pointer to IMMR in r4 */
	addis    r4,0,IMMR_HI+1
	
	bl	init_flash

	bl       init_siu

	mtspr    LR, r31
        blr      	
	
/************************************************************************
*
* Function: Copy_code
*
* Description: This function copies the flash content to the sdram
*              and changes the memory configuration.
*
************************************************************************/

copy_code:	
	mfspr    r31,LR

	/* Copy first megabyte of Flash to SDRAM */
	addis	r3,0,SDRAM_BR_BOOT@h
	addis	r4,0,FLASH_BR_BOOT@h
	addis	r5,0, _end@h
	ori     r5,r5,_end@l
	bl	copy_mem

	/* Keep pointer to IMMR in r4 */
	addis    r4,0,IMMR_HI+1
	
	/* Swap SDRAM to lowmem and FLASH to highmem */
	addis	r3,0, SDRAM_BR@h
	ori     r3,r3, SDRAM_BR@l
	stw	r3, BR1(r4)
	
	addis   r3,0, FLASH_BR@h
	ori     r3,r3,FLASH_BR@l
	stw	r3,BR0(r4)
	
	bl	EnableCacheEtc
			
	mtspr    LR, r31
        blr      	
	

/************************************************************************
* Function: init_siu
*
* Description: Initializes the System Interface Unit
*
/***********************************************************************/

init_siu:
	mfspr    r30, LR

	/* 60x compatible bus + pipeline depth zero + non MPC8260 bustmasters */
	addis    r3,0,0x8080
	ori      r3,r3,0xe000
	stw      r3,BCR(r4)

	 /* ... 0x4000 = External snoop enable, */	
	addis    r3,0,(HWC_DPPC<<10)+(HWC_L2CPC<<8)+(HWC_APPC<<4)+(HWC_CS10PC<<2) + 0x4000
	ori      r3,r3,0x0000
	stw      r3,SIUMCR(r4)

#if 0	/* Watchdog is enabled/disabled wdt_init() to be called from _init() */
	addis    r3,0,0xFFFF
#ifndef WATCHDOG_DISABLED
	ori      r3,r3,0xFFc7		/* Watchdog & Busmonitors enable */
#else
	ori      r3,r3,0xFF00		/* Watchdog & Busmonitors disabled */
#endif
	stw      r3,SYPCR(r4)
#endif

	addi     r3,0,0x0002
	stb      r3,PPC_ACR(r4)

	addis    r3,0,0x0126
	ori      r3,r3,0x7893
	stw      r3,PPC_ALRH(r4)

        addis    r3,0,0x89ab
        ori      r3,r3,0xcdef
        stw      r3,PPC_ALRL(r4)

        li       r3,0x02
        stb      r3,LCL_ACR(r4)

        addis    r3,0,0x0126
        ori      r3,r3,0x7983
        stw      r3,LCL_ALRH(r4)		

        addis    r3,0,0x89ab
        ori      r3,r3,0xcdef
        stw      r3,LCL_ALRL(r4)

	addis    r3,0,0x0000
	ori      r3,r3,0x4000
	stw      r3,TESCR1(r4)

	addis    r3,0,0x0000
	ori      r3,r3,0x4000
	stw      r3,LTESCR1(r4)

	addis    r3,0,0x0000
	ori      r3,r3,0x0000	
	stw      r3,SCCR(r4)

	mtspr    LR, r30
	blr


/***********************************************************************
* Function: init_flash
*
* Description: This function programs Base Register 0 and Option Register 0
*
************************************************************************/

init_flash:
	/* 1st flash */

	addis    r3,0,FLASH_BR_BOOT@h
	ori      r3,r3,FLASH_BR_BOOT@l
	
	addis    r5,0,FLASH_OR@h
	ori      r5,r5,FLASH_OR@l

	/*
	  It's important to note the order in which OR0 and BR0 are programmed.
	  When coming out of reset and CS0 is the global chip select, OR0 MUST be 
	  programmed AFTER BR0. In all other cases BRx would be programmed after
	  ORx.
	*/
	stw	r3,BR0(r4)
	stw	r5,OR0(r4)
	
	blr

/*
 * Initialize SDRAM memory controller
 */	
		
init_sdram:
	/* Keep pointer to IMMR in r4 */
	addis    r4,0,IMMR_HI+1

	/* Option Register 2 */
	addis    r5,0, SDRAM_OR@h
	ori      r5,r5, SDRAM_OR@l
	stw      r5,OR1(r4)
	
	/* Base Register 2 */
	addis	r3,0, SDRAM_BR_BOOT@h
	ori     r3,r3, SDRAM_BR_BOOT@l
	stw     r3,BR1(r4)

	/*  Memory Refresh Timer Prescaler Register (10.3.12): 50 */
	addi	r5,0,0x3200      
        sth     r5,MPTPR(r4)    

	/* 60x bus-assigned SDRAM refresh timer (10.3.10):  PSRT = 14 (66MHz bus clock (?)) */
	addi	r5, 0, 0x0e
	stb	r5, PSRT(r4)

	/* Precharge  all */
	
	addis   r3,0,SD_PSDMR+SDRAM_OP_PRECHARGE
	ori     r3,r3,SD_PSDMR_LO
	stw     r3,PSDMR(r4)

	addis   r5,0, SDRAM_BR_BOOT@h
	li      r3,0             /* Load 0x000000FF into r3 */
	stb     r3,0(r5)         /* Write 0xFF to address 0 - bits [24-31] */	

	/* Start refresh */
		
	addis   r3,0,SD_PSDMR+SDRAM_OP_CBRREFR
	ori     r3,r3,SD_PSDMR_LO
	stw	r3,PSDMR(r4)

	li	r3,0x00
	stb	r3,0(r5)
	stb	r3,1(r5)
	stb	r3,2(r5)
	stb	r3,3(r5)
	stb	r3,4(r5)
	stb	r3,5(r5)
	stb	r3,6(r5)
	stb	r3,7(r5)

	/* Write mode register */
			 
	addis   r3,0,SD_PSDMR+SDRAM_OP_MODEWR
	ori     r3,r3,SD_PSDMR_LO
	stw     r3,PSDMR(r4)

	li	r3,0xff          
	stb	r3,((((SDRAM_CL<<4)+ SDRAM_BURSTLEN)<<3)+0)(r5)
	
	/* go to Normal operation */
	
	addis   r3,0,SD_PSDMR+SDRAM_OP_NORMAL
	ori     r3,r3,SD_PSDMR_LO
	stw     r3,PSDMR(r4)
			
	blr
	
/*
 * Initialize stack pointer
 */

init_ministack:

	/* Exception stack uses same memory area as normal stack.
	   Exceptions should not happend during initialization.
	*/
	addis    r1,r0,IMMR_HI
	ori      r1,r1,0x3000
	mtspr	SPRG1, r1

	addis    r1,r0,IMMR_HI
	ori      r1,r1,0x4000

	/* load the stack pointer 72 bytes down from top of stack to ensure one
	   stack frame of safety margin. 
	*/

	stwu	r0,-72(r1)      
	                         
	bclr	20,0

/*
 * Initialize stack pointer
 */
	.globl init_stack

init_stack:

	/* Initialize exception stack pointer (SPRG1) first */
	addis    r1,r0,__ESP_INIT@h  
	ori      r1,r1,__ESP_INIT@l
	mtspr	SPRG1, r1

	/* SP_INIT this value is defined in init8260.lnk which is the linker file */
	addis    r1,r0,__SP_INIT@h  
	ori      r1,r1,__SP_INIT@l

	/* load the stack pointer 72 bytes down from top of stack to ensure one
	   stack frame of safety margin. 
	*/

	stwu	r0,-72(r1)      
	                         
	bclr	20,0

/*
 * Enable instruction cache + Branch prediction + Store Gathering
 */	

EnableCacheEtc:
	         
	mfspr	r5,HID0        
	ori	r5,r5,0x88a4   /* Set the ICE and ICFI bit + BHT + BTIC + SGE */
	andi.	r6,r5,0xF7FF   /* clear the ICFI bit for the final store */
	mtspr	HID0,r5
	isync
	sync

	mtspr	HID0,r6
	isync
	sync
	bclr	20,0   

	
/*
 * Delay for a number of microseconds
 * -- Use the BUS timer (assumes 66MHz)
 */
        .globl  udelay
udelay:         
        mulli   r4,r3,1000      /* nanoseconds */
        addi    r4,r4,59
        li      r5,60
        divw    r4,r4,r5        /* BUS ticks */
1:      mftbu   r5
        mftb    r6
        mftbu   r7
        cmp     0,r5,r7
        bne     1b              /* Get [synced] base time */
        addc    r9,r6,r4        /* Compute end time */
        addze   r8,r5
2:      mftbu   r5
        cmp     0,r5,r8
        blt     2b
        bgt     3f
        mftb    r6
        cmp     0,r6,r9
        blt     2b
3:      blr             

	.globl utime		/* Return timebase lo */
utime:	
        mftb    r3
	blr

	.globl utimehi		/* Return timebase lo */
utimehi:	
        mftbu   r3
	blr

	
/* r3=dstaddr r4=srcaddr r5=count */
copy_mem:
        rlwinm. r7,r5,32-2,2,31         /* r7 = r5 >> 2 */
	addi	r3, r3, -4
	addi	r4, r4, -4
	mtctr	r7
1:	lwzu	r7, 4(r4)
	stwu	r7, 4(r3)
	bdnz	1b
	blr
	
       /* Initialize LED's for entertainment only */
init_leds:
	addis	 r4, 0, IMMR_HI+1
 	addis	 r3, 0, SIPO_LEDS_PORTA_MASK@h
	ori	 r3, r3, SIPO_LEDS_PORTA_MASK@l
	lwz	 r5, PDIRA(r4)
	or	 r5, r5, r3	
	stw      r5, PDIRA(r4)
	lwz	 r5, PDATA(r4)
	or	 r5, r5, r3
	stw	 r5, PDATA(r4)

 	addis	 r3, 0, SIPO_LEDS_PORTC_MASK@h
	ori	 r3, r3, SIPO_LEDS_PORTC_MASK@l
	lwz	 r5, PDIRC(r4)
	or	 r5, r5, r3	
	stw      r5, PDIRC(r4)
	lwz	 r5, PDATC(r4)
	or	 r5, r5, r3
	stw	 r5, PDATC(r4)

	blr
	

       /* set leds */
	.globl set_leds
set_leds:
	nor	r3, r3, r3
	addis	r4, 0, IMMR_HI+1
	addis	r5, 0, SIPO_LEDS_PORTC_MASK@h
	ori	r5, r5, SIPO_LEDS_PORTC_MASK@l
	and	r5, r5, r3
        stw     r5, PDATC(r4)
	addis	r5, 0, SIPO_LEDS_PORTA_MASK@h
	ori	r5, r5, SIPO_LEDS_PORTA_MASK@l
	and	r5, r5, r3	
        stw     r5,PDATA(r4)
	blr

.end	/* end of .text */

