/*********************************************************************/
/*                                                                   */
/* Printf                                                            */
/*                                                                   */
/*********************************************************************/

#include <stdarg.h>
#include "stdio.h"
#include "string.h"
#include "ctype.h"

static int vsprintf(char *s, const char *format, va_list arg);

int printf(const char *format, ...)
{
    char buf[2048];
    va_list arg;
    int i;
    char    *cp;

    va_start(arg, format);
    i = vsprintf(buf, format, arg);
    va_end(arg);

    for(cp = buf; i > 0; i--) 
    {
        serial_putc(*cp);
        if (*cp == '\n')
            serial_putc('\r');
        cp++;
    }

    return cp-buf;
}


int sprintf(char *s, const char *format, ...)
{
    va_list arg;
    int ret;

    va_start(arg, format);
    ret = vsprintf(s, format, arg);
    va_end(arg);
    return (ret);
}

#define ZERO    1
#define SIGN    2
#define PLUS    4
#define SPACE   8
#define LEFT    16

static int vsprintf(char *str, const char *format, va_list arg)
{
    int           flags,width,qual,base,prec,neg;
    long          lvalue;
    unsigned long ulvalue;
    char *s,*t;
    char tmp[66];
    int i,j;


    for(s=str; *format; format++) 
    {
        if(*format!='%') {
            *s++=*format;
            continue;
        }
        flags=0;
        while(1) {
            format++;
            if(*format=='-') flags |= LEFT;
            else if(*format=='+') flags |= PLUS;
            else if(*format==' ') flags |= SPACE;
            else if(*format=='0') flags |= ZERO;
            else break;
        }

        /* get width */
        for(width=0; isdigit((unsigned char)*format);)
            width = width * 10 + (*format++ - '0');

        /* get precision */
        prec = -1;
        if (*format == '.') {
            ++format;  
            while (isdigit((unsigned char)*format))
                prec = prec * 10 + (*format++ - '0');
            if (prec < 0)
                prec = 0;
        }

        if (*format == 'h' || *format == 'l' || *format == 'L')
            qual = *format++;
        else
            qual = -1;

        base = 10;

        switch(*format) {
        case 'c':
            while (!(flags & LEFT) && --width > 0)
                *s++ = ' ';
            *s++ = (unsigned char) va_arg(arg, int);
            while (--width > 0)
                *s++ = ' ';
            continue;

        case 's':
            t = va_arg(arg, char *);

            i = strlen(t);

            while (!(flags & LEFT) && i < width--)
                *s++ = ' ';
            for (j = 0; j < i; j++)
                *s++ = *t++;
            while (i < width--)
                *s++ = ' ';
            continue;

        case '%':
            *s++ = '%';
            continue;

        case 'b':
            base = 2;
            break;

        case 'o':
            base = 8;
            break;

        case 'X':
        case 'x':
            base = 16;
            break;

        case 'd':
        case 'i':
            flags |= SIGN;
        case 'u':
            break;

        default:
            *s++ = '%';
            if (*format)
                *s++ = *format;
            else
                --format;
            continue;
        }
        if (qual == 'l')
            lvalue = va_arg(arg, long);
        else if (qual == 'h')
            lvalue = va_arg(arg, short);
        else
            lvalue = va_arg(arg, int);

        /* Print a number ... */

        if((flags&SIGN) && (lvalue<0)) {
            ulvalue = -lvalue;
            neg=1;
        } else {
            ulvalue = (unsigned long)lvalue;
            neg=0;
        }
        i=0;
        if(ulvalue==0)
            tmp[i++]='0';
        while(ulvalue>0) 
        {
            j = ulvalue%base;
            if(j<10) 
                tmp[i] = '0' + j;
            else
                tmp[i] = 'a' + j - 10;
            ulvalue /= base;
            i++;
        }
        if (i > prec)
                prec = i;
        width -= prec;

        while(!(flags&(ZERO+LEFT)) && width-->0)
            *s++ = ' ';

        if(flags&SIGN) {
            if(neg) 
                *s++='-';
            else if(flags&PLUS)
                *s++='+';
            else if(flags&SPACE)
                *s++=' ';
            else
                width++;
            width--;
        }

        while (!(flags & LEFT) && width-- > 0)
            *s++ = (flags & ZERO) ? '0' : ' ';

        while (i < prec--)
                *s++ = '0';

        while (i-- > 0)
                *s++ = tmp[i];

        while (width-- > 0)
                *s++ = ' ';     
    }
    *s = '\0';
    return s-str;
}

