
/*------------------------------------------------------------------------*
*  FILE:  MASK8260.H                                                     
*                                                                        
*  DESCRIPTION:  Bit masks for MPC8260 registers. Sequence of registers  
*                follows device memory map exactly.                      
*                                                                         
*  HISTORY:  8 JAN 99 JAY                                         
*------------------------------------------------------------------------*/

/*------------------------------------------*
* CPM Command Register (CPCR)               *
*-------------------------------------------*
* NOTE: This register is cleared by reset.  *
*       See MPC8260 User's Manual.          *
*-------------------------------------------*/

#define CPCR_RST                 0x80000000  /* Software Reset Command */
#define CPCR_FLG                 0x00010000  /* Command Semaphore Flag */

/*-----------------------------------------------*/
/* Definitions for SCC CPCR Subblock/Page codes. */
/*-----------------------------------------------*/

#define  SCC1_PAGE_SUBBLOCK    0x00800000    /* page 0, code = 4 */
#define  SCC2_PAGE_SUBBLOCK    0x04A00000    /* page 1, code = 5 */
#define  SCC3_PAGE_SUBBLOCK    0x08C00000    /* page 2, code = 6 */
#define  SCC4_PAGE_SUBBLOCK    0x0CE00000    /* page 3, code = 7 */

/*-----------------------------*/
/* Opcode definitions for SCCs */
/*-----------------------------*/

#define CPCR_INIT_TX_RX_PARAMS   0x00000000   /* Opcode 0 */
#define CPCR_INIT_RX_PARAMS      0x00000001   /* Opcode 1 */
#define CPCR_INIT_TX_PARAMS      0x00000002   /* Opcode 2 */
#define CPCR_ENTER_HUNT_MODE     0x00000003   /* Opcode 3 */
#define CPCR_STOP_TX             0x00000004   /* Opcode 4 */
#define CPCR_GRACEFUL_STOP_TX    0x00000005   /* Opcode 5 */
#define CPCR_RESTART_TX          0x00000006   /* Opcode 6 */
#define CPCR_CLOSE_RX_BD         0x00000007   /* Opcode 7 */
#define CPCR_SET_GRP_ADDR        0x00000008   /* Opcode 8 */
#define CPCR_RESET_BCS           0x0000000A   /* Opcode 10 */


/*-------------------------*/
/* General SCC Definitions */
/*-------------------------*/

#define  DISABLE_TX_RX   0xFFFFFFCF  /* Clear the ENT/ENR bits in the GSMR
                                        Disables the transmit & Receive
                                        port                              */

#define  GSMR_L1_ENT  0x00000010     /* ENT bit for the GSMR low register */
#define  GSMR_L1_ENR  0x00000020     /* ENR bit for the GSMR low register */


