#define NET_BOOT_MAX_REPEAT 3

#define NULL ((void *)0)
#define TRUE 1
#define FALSE 0

#define PROT_IP         0x0800
#define PROT_ARP        0x0806
#define PROT_IP_UDP     17

#define ARP_REQUEST     1
#define ARP_REPLY       2
#define ARP_MAX_REPEAT  5

#define BOOTP_REQUEST   1
#define BOOTP_REPLY     2
#define BOOTP_MAX_REPEAT 5

#define TFTP_DEFAULTSIZE_PACKET 512
#define TFTP_MAX_PACKET         1432 /* 512 */
#define TFTP_MAX_REPEAT         20

#define TFTP_RRQ        1
#define TFTP_WRQ        2
#define TFTP_DATA       3
#define TFTP_ACK        4
#define TFTP_ERROR      5
#define TFTP_OACK       6

#define TFTP_CODE_EOF   1
#define TFTP_CODE_MORE  2
#define TFTP_CODE_ERROR 3
#define TFTP_CODE_BOOT  4
#define TFTP_CODE_CFG   5

#define BOOTP_SERVER    67
#define BOOTP_CLIENT    68
#define TFTP_PORT       69

#define RFC1533_COOKIE          99, 130, 83, 99
#define RFC1533_NETMASK         1
#define RFC1533_GATEWAY         3
#define RFC1533_DNS             6
#define RFC1533_HOSTNAME        12
#define RFC1533_BOOTFILESIZE    13
#define RFC1533_ROOTPATH        17
#define RFC1533_NISDOMAIN       40
#define RFC1533_END             255

#define BOOTP_VENDOR_LEN        64
#define DHCP_OPT_LEN            312


typedef unsigned char ethaddr[6];
typedef unsigned int in_addr;

struct arptable_t {
    in_addr ipaddr;
    ethaddr ethaddr;
};

typedef struct {
    ethaddr dstaddr;
    ethaddr srcaddr;
    unsigned short type;
} ethhdr;

typedef struct {
    unsigned short hwtype;
    unsigned short protocol;
    char hwlen;
    char protolen;
    unsigned short opcode;
    char shwaddr[6];
    char sipaddr[4];
    char thwaddr[6];
    char tipaddr[4];
} arprequest;

typedef struct {
    char verhdrlen;
    char service;
    unsigned short len;
    unsigned short ident;
    unsigned short frags;
    char ttl;
    char protocol;
    unsigned short chksum;
    in_addr src;
    in_addr dest;
} iphdr;

typedef struct {
    unsigned short src;
    unsigned short dest;
    unsigned short len;
    unsigned short chksum;
} udphdr;

typedef enum {
    SOCK_FREE=0,
    SOCK_READY,
    SOCK_INVALID
} sock_state;

/* UDP sock flags */
#define UDP_SOCK_CHECKSUM 1

typedef struct {
    in_addr     src_ip;
    in_addr     dst_ip;
    unsigned short src_port;
    unsigned short dst_port;
    void        *data;
    sock_state state;
    unsigned int flags;
} udp_socket;

typedef struct {
    char op;
    char htype;
    char hlen;
    char hops;
    unsigned long xid;
    unsigned short secs;
    unsigned short unused;
    in_addr ciaddr;
    in_addr yiaddr;
    in_addr siaddr;
    in_addr giaddr;
    char hwaddr[16];
    char sname[64];
    char file[128];
    char vend[DHCP_OPT_LEN];
} bootp_t;

typedef struct {
    unsigned short opcode;
    union {
        char rrq[TFTP_DEFAULTSIZE_PACKET];
        char wrq[TFTP_DEFAULTSIZE_PACKET];
        struct {
            unsigned short block;
            char download[TFTP_MAX_PACKET];
        } data;
        struct {
            unsigned short block;
        } ack;
        struct {
            unsigned short errcode;
            char errmsg[TFTP_DEFAULTSIZE_PACKET];
        } err;
        struct {
            char data[TFTP_DEFAULTSIZE_PACKET+2];
        } oack;
    } u;
} tftp_t;


extern in_addr MY_IPADDR;
extern in_addr MY_NETMASK;
extern in_addr MY_GATEWAY;
extern in_addr MY_DNSSERV;
extern char    MY_NAME[];
extern in_addr SERVER_IPADDR;
extern char    rootpath[];
extern char    bootfile[];

int bootp();
int tftp_get(char *buffer, char *filename, in_addr serverip);
int tftp_put(char *buffer, int len, char *filename, in_addr serverip);

