#include "ethernet.h"
#include "fcc.h"
#include "net.h"
#include "netcomm.h"
#include "mpc8260.h"
#include "mask8260.h"
#include "board.h"

#define SIPO_LED_BLUE0          (0x80000000>>22)
#define SIPO_LED_BLUE1          (0x80000000>>23)
#define SIPO_LED_BLUE2          (0x80000000>>24)
#define SIPO_LED_BLUE3          (0x80000000>>25)
#define SIPO_LED_STATUS_GREEN   (0x80000000>>7)
#define SIPO_LED_STATUS_RED     (0x80000000>>8)
#define SIPO_LED_STATUS_WEIRD   (SIPO_LED_STATUS_RED|SIPO_LED_STATUS_GREEN)
#define SIPO_LED_POWER          (0x80000000>>6)
#define SIPO_LEDS_PORTA_MASK    (SIPO_LED_BLUE0|SIPO_LED_BLUE1|SIPO_LED_BLUE2|SIPO_LED_BLUE3)
#define SIPO_LEDS_PORTC_MASK    (SIPO_LED_STATUS_WEIRD|SIPO_LED_POWER)

#define KERNEL_LOAD_ADDRESS     0x0d00000

#define FLASH_MAX_RETRY 5

#define SIPO_HZ 10                              /* Timer tics 20 times per second */
#define SIPO_CLI_COUNTDOWN      (300*SIPO_HZ)   /* 300 seconds coundown to reboot after last command */
#define SIPO_COUNTDOWN          (30*SIPO_HZ)    /* Default countdown after reset */

extern sipo_board_info *__bd;

extern char _end[];                             /* End of .text */
extern char __HEAP_BEGIN[];                     /* Beginning of free memory */

#define SIPO_VERSION_STRING "SIPO bootloader <special> v1.3"
#define SIPO_COPYRIGHT_STRING "Copyright (C) 2000-2002 SSH Communications Security Corp, Finland"

extern void *malloc(unsigned);
extern void free(void *);

extern int flash_erase_block(unsigned int blocknum);
extern int flash_program_block(unsigned int blocknum, void *addr);
extern int flash_program_block_jffs2_hdr(unsigned int blocknum);
extern void flash_clear_lock_bits();
extern long long flash_read_serial_number(unsigned char *Manuf_ID, unsigned char *Device_Type, unsigned char *Lock, char *SerialLock);
extern void flash_write_serial_number(long long SerialNr);
extern int flash_write(int blocknum, void *addr, unsigned int count);
extern void flash_test(void);
extern int flash_cutcopypaste(unsigned int offset, void *addr, unsigned int count);

extern int sdram(void);
extern int memtest(volatile unsigned *base, int size);
extern unsigned pattern(unsigned int s);

extern char *readline(char *, int);
extern char *readpasswd(char *pwd, int n);
extern int partition_set(int index, sipo_partition_type type, int offset, int size);
extern void partition_print();
extern void print_bd(void);
extern int flash_tftp(char *fname, int ix, int skip_verify);
extern int boot(char *cmd);
extern int net_boot(char *cmd);
extern int test_spd(char *cpp);

extern void rtc_init();
extern void rtc_write(unsigned int address, unsigned int len, unsigned char *buffer);
extern void rtc_read(unsigned int address, unsigned int len, unsigned char *buffer);
extern int nv_write(unsigned int address, unsigned int len, unsigned char *buffer);
extern int nv_read(unsigned int address, unsigned int len, unsigned char *buffer);
extern void print_rtc_time();
extern int set_rtc_time(int sec, int min, int hour, int day, int mon, int year);

typedef struct {
    unsigned long state[5];
    unsigned long count[2];
    unsigned char buffer[64];
} SHA1_CTX;

extern void SHA1Init(SHA1_CTX* context);
extern void SHA1Update(SHA1_CTX* context, unsigned char* data, unsigned long len);
extern void SHA1Final(unsigned char *digest, SHA1_CTX* context);
extern void sha1(void *buffer, int len, SHA1Digest digest);
extern int verify(void *buffer, int len);

extern unsigned int wdt_countdown;

extern void set_leds(int leds);

#define mb()  __asm__ __volatile__ ("sync" : : : "memory")

#define RUNNING_IN_FLASH ((IMMR->memc_regs[0].br&0xffff0000)==0)

