 .text
 .align 2


 .globl  Xreset       
 .globl  VectorTable  
 .globl  VectorTableEnd
 .globl  __Xitlbmiss         


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/**/
/*   Exception Vector Table. Note: This is the starting point for code*/
/*   mapping. Note that "handler" is called in some of the vectors. This */
/*   interrupt function has been created in template form as a starting */
/*   point for the user. It is defined further down in this module.*/
/**/
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

VectorTable:

Xreset:

   /*------------------------------------------------------------------------*/
   /* Reprogram Option Register 0 so that chip select 0 (CS0) will respond to */
   /* the complete 4 GByte address range. Where the code is located in the */
   /* memory map will not be an issue until it is definitively defined in the*/
   /* init_memc function.*/
   /*------------------------------------------------------------------------*/

   addis    r4,0,IMMR_HI+1
   addis    r5,0,0		
   ori      r5,r5, FLASH_OR@l
   stw      r5,OR0(r4)

   /*------------------------------------------------------------------------*/
   /* This isync was added to force the core from executing or pre-fetching*/
   /* any more instructions before the previous one to give the memory */
   /* controller a chance to re-program the Option Register 0 logic.*/
   /*------------------------------------------------------------------------*/

   isync

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/
 
   addis    r2,0,_start@h    
   ori      r2,r2,_start@l
   mtspr    LR,r2 

   blr
	
   /*~~~~~~~~~~~~~~~~~~~~~~                                          */
   /* 0200: MACHINE CHECK               */
   /*~~~~~~~~~~~~~~~~~~~~~~                                          */

   .skip 0x0100-(.-VectorTable)

__Xmchk:

   /*--------------------------------------------*/
   /* save off registers used in vector routine */
   /*--------------------------------------------*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*------------------------------------------------------------------*/
   /* Set the ME bit so that additional machine checks will not cause*/
   /* a checkstop state.*/
   /*------------------------------------------------------------------*/

   mfmsr  r3
   ori    r3,r3,0x1000
   mtmsr  r3
   isync

   /*-------------------------------------#*/
   /*   FLUSH AND INVALIDATE THE CACHES   # */
   /*-------------------------------------#*/

   mfspr r3,SRR0  /* get the faulting instruction address*/
   icbi  0,r3     /* invalidate the cache block*/
   mfspr r3,DAR   /* get the faulting data from DAR */
   dcbi  0,r3     /* invalidate the data cache block*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0002
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~~~~~~~~~~~~~                                          */
   /* 0300: DATA ACCESS ERROR              */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~                                          */

   .skip 0x0200-(.-VectorTable)

__data_access_err:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0003
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
 

   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                          */
   /* 0400:INSTRUCTION ACCESS ERROR          */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                          */

    .skip 0x0300-(.-VectorTable)

__inst_access_error:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0004
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
 
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~                                          */
   /* 0500: EXTERNAL INTERRUPT             */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~                                          */

   .skip 0x0400-(.-VectorTable)

__Xexint:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0005
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~~~~~~~~~~                                          */
   /* 0600: ALIGNMENT ERROR             */
   /*~~~~~~~~~~~~~~~~~~~~~~                                          */

   .skip 0x0500-(.-VectorTable)

__Xalgn:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0006
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
 
   /*~~~~~~~~~~~~~~~~~~~~~~                                          */
   /* 0700: PROGRAM ERROR               */
   /*~~~~~~~~~~~~~~~~~~~~~~                                          */

    .skip 0x0600-(.-VectorTable)

__Xprog:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0007
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                          */
   /* 0800: FLOATING-POINT UNAVAILABLE           */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                          */

    .skip 0x0700-(.-VectorTable)

__float_unavailable:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0008
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
 
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                          */
   /* 0900: DECREMENTER INTERRUPT          */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                          */

   .skip 0x0800-(.-VectorTable)

__Xdec:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0009
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~~~                                          */
   /* 0A00: RESERVED              */
   /*~~~~~~~~~~~~~~~                                          */
   

   /*~~~~~~~~~~~~~~~                                          */
   /* 0B00: RESERVED              */
   /*~~~~~~~~~~~~~~~                                          */
   

   /*~~~~~~~~~~~~~~~~~~~                                          */
   /* 0C00: SYSTEM CALL                 */
   /*~~~~~~~~~~~~~~~~~~~                                          */

   .skip 0x0B00-(.-VectorTable)

__Xsysc:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x000C
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~                                          */
   /* 0D00: TRACE                 */
   /*~~~~~~~~~~~~~                                          */

   .skip 0x0C00-(.-VectorTable)

__Xtrace:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x000D
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~~~~                                          */
   /* 0E00: RESERVED           */
   /*~~~~~~~~~~~~~~~~                                          */


   /*~~~~~~~~~~~~~~~                                          */
   /* 0F00: RESERVED              */
   /*~~~~~~~~~~~~~~~                                          */
   

   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                          */
   /* 1000:INSTRUCTION TLB MISS               */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

   .skip 0x0F00-(.-VectorTable)

__Xitlbmiss:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0010
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
   /* 1100: DATA LOAD TRANSLATION MISS         */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

   .skip 0x1000-(.-VectorTable)

__Xdltlbmiss:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0011
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/



   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
   /* 1200:DATA STORE TRANSLATION MISS          */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

   .skip 0x1100-(.-VectorTable)

__Xdstlbmiss:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0012
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
   /* 1300:INSTRUCTION ADDRESS BREAKPOINT          */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

   .skip 0x1200-(.-VectorTable)

__Xiaddrbrk:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0013
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
   /* 1400: SYSTEM MANAGEMENT INTERRUPT          */
   /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

   .skip 0x1300-(.-VectorTable)

__Xsysmgmtint:

   /*-------------------------------------------#*/
   /* save off registers used in vector routine #*/
   /*-------------------------------------------#*/

   mtspr SPRG3,r3    /* save r3*/
   
   mfspr r3,LR
   mtspr SPRG2,r3    /* save LR*/

   /*-----------------------------------------------------------------------*/
   /* Load the vector offset value in SPRG0 for handler shifted down 8 bits.*/
   /*----------------------------------------------------------------------- */

   addi  r3,r0,0x0014
   mtspr SPRG0,r3

   /*---------------------------------------------------------*/
   /* load link register in order to jump to physical address*/
   /*---------------------------------------------------------*/

   addis    r3,0,handler@h    
   ori      r3,r3,handler@l
   mtspr    LR,r3 
   bclr     20,0                 /* jump unconditionally to address in Link */
                                 /* Register (LR)*/


   /*~~~~~~~~~~~~~~~~~~~~~                                          */
   /* 1500-2FFF: RESERVED              */
   /*~~~~~~~~~~~~~~~~~~~~~                                          */
   
   /*------------------------------------------------------------------------*/
   /* Force the assembler to fill all zeros to the end the vector table area.*/
   /* Put a noop instruction at the last 4 bytes of the table to accomplish*/
   /* this. Immediately after this instruction will be _start:*/
   /*------------------------------------------------------------------------*/

   .skip 0x2EFC-(.-VectorTable)

   ori   r0,r0,0     /* instruction that does nothing*/


VectorTableEnd:

/*******************************************************************************

	Interrupt handler code		

	
*******************************************************************************/

 .globl     handler

	
handler:
         
	/* Restore registers used by vector routine */

	mfspr	r3,SPRG2
	mtspr	LR,r3
	mfspr	r3,SPRG3    /*# Restore the original value of R3*/

	/* Save temporary registers */	
	mtspr	SPRG2, r20
	mtspr	SPRG3, r21

	/* R21 is used here as exeption stack pointer */
	mfspr	r21, SPRG1
	subi	r21, r21, INT_FRAME_SIZE

	/* Save registers */
	mfcr	r20
	stw     r20,_CCR(r21)
        stw     r22,GPR22(r21)
        stw     r23,GPR23(r21)
        mfspr   r20,SPRG2
        stw     r20,GPR20(r21)
        mfspr   r22,SPRG3
        stw     r22,GPR21(r21)
        mflr    r20
        stw     r20,_LINK(r21)
        mfctr   r22
        stw     r22,_CTR(r21)
        mfspr   r20,XER
        stw     r20,_XER(r21)
        mfspr   r22,SRR0
        stw     r22,_SRR0(r21)
        mfspr   r23,SRR1
        stw     r23,_SRR1(r21)
        stw     r0,GPR0(r21)
        stw     r1,GPR1(r21)
        stw     r2,GPR2(r21)
        stw     r1,0(r21)
        SAVE_4GPRS(3, r21)
        SAVE_GPR(7, r21)
        SAVE_4GPRS(8, r21)
        SAVE_8GPRS(12, r21)
        SAVE_8GPRS(24, r21)

	or	r1, r21, r21
	
	/*#-------------------------------------------------------------------------*/
	/*# The following code uses the exception value (shifted to the right 8 bits*/
	/*# in vector_tbl.s) in SPRG0 as a multiplier from jump_tbl_addr: times the */
	/*# number of bytes in the bl/b instruction combination in order to vector */
	/*# to the proper branch to the final interrupt handler for the exception */
	/*# that occured. This method gives a low interrupt latency. */
	/*#-------------------------------------------------------------------------*/

	mfspr	r0, SPRG0
	mfspr	r3, SPRG0	/* Save it to R3 */
	mulli	r0, r0, 0x0008 /* multiply shifted exception value by number of */

	addis	r2,0,jump_tbl_addr@ha
	ori	r2,r2,jump_tbl_addr@l
	add	r0,r0,r2
	mtspr	LR,r0

	bclr	20,0        


finish_exception:

        addi    r0,r1,INT_FRAME_SIZE    /* size of frame */
        mtspr   SPRG1,r0                /* Store stack pointer */

	/* Restore registers */
        lwz     r3,_XER(r1);  
        mtspr   XER, r3;        
	lwz     r3,_CCR(r1);
	mtcrf	0xff, r3
        lwz     r3,_CTR(r1)
        mtctr   r3
        lwz     r3,_LINK(r1)
        mtlr    r3
        REST_GPR(3, r1)
        REST_10GPRS(4, r1)
        REST_10GPRS(14, r1)
        REST_8GPRS(24, r1)
        lwz     r0,_SRR0(r1)
        lwz     r2,_SRR1(r1)
        mtspr   SRR1,r2
        mtspr   SRR0,r0
        lwz     r0,GPR0(r1)
        lwz     r2,GPR2(r1)
        lwz     r1,GPR1(r1)
        sync
        rfi


	
   /*#######################*/
   /*# Exception Jump Table*/
   /*#######################*/

jump_tbl_addr:

   /*#------------------*/
   /*# Reserved - 00000*/
   /*#------------------*/

   bl unknown_exception
   b finish_exception

   /*#-------------------------*/
   /*# Reset Exception - 00100*/
   /*#-------------------------*/

   b unknown_exception
   b finish_exception

   /*#---------------------------------*/
   /*# Machine Check Exception - 00200*/
   /*#---------------------------------*/

   bl machine_check_handler
   b finish_exception

   /*#------------------------------------------------*/
   /*# Data Storage Interrupt (DSI) Exception - 00300*/
   /*#------------------------------------------------*/

   bl dsi_handler
   b finish_exception

   /*#---------------------------------------*/
   /*# Instruction Storage Exception - 00400*/
   /*#---------------------------------------*/

   bl isi_handler
   b finish_exception

   /*#----------------------------*/
   /*# External Exception - 00500*/
   /*#----------------------------*/

   bl external_handler
   b  finish_exception

   /*#-----------------------------*/
   /*# Alignment Exception - 00600*/
   /*#-----------------------------*/

   bl alignment_handler
   b  finish_exception

   /*#---------------------------*/
   /*# Program Exception - 00700*/
   /*#---------------------------*/

   bl program_handler
   b  finish_exception

   /*#----------------------------------------------*/
   /*# Floating Point Unavailable Exception - 00800*/
   /*#----------------------------------------------*/

   bl float_unavail_handler
   b  finish_exception

   /*#-------------------------------*/
   /*# Decrementer Exception - 00900*/
   /*#-------------------------------*/

   bl timer_interrupt
   b  finish_exception

   /*#----------------------------*/
   /*# Reserved Exception - 00A00*/
   /*#----------------------------*/

   bl unknown_exception
   b  finish_exception

   /*#----------------------------*/
   /*# Reserved Exception - 00B00*/
   /*#----------------------------*/

   bl unknown_exception
   b  finish_exception

   /*#-------------------------------*/
   /*# System Call Exception - 00C00*/
   /*#-------------------------------*/

   bl system_call_handler
   b  finish_exception

   /*#-------------------------*/
   /*# Trace Exception - 00D00*/
   /*#-------------------------*/

   bl trace_handler
   b  finish_exception

   /*#----------------------------*/
   /*# Reserved Exception - 00E00*/
   /*#----------------------------*/

   bl unknown_exception
   b  finish_exception

   /*#----------------------------*/
   /*# Reserved Exception - 00F00*/
   /*#----------------------------*/

   bl unknown_exception
   b  finish_exception

   /*#------------------------------------------------*/
   /*# Instruction Translation Miss Exception - 01000*/
   /*#------------------------------------------------*/

   bl inst_trans_miss_handler
   b  finish_exception

   /*#----------------------------------------------*/
   /*# Data Load Translation Miss Exception - 01100*/
   /*#----------------------------------------------*/

   bl data_load_miss_handler
   b  finish_exception

   /*#----------------------------------------------*/
   /*# Data Store Translation Miss Exception - 01200*/
   /*#----------------------------------------------*/

   bl data_store_miss_handler
   b  finish_exception

   /*#--------------------------------------------------*/
   /*# Instruction Address Breakpoint Exception - 01300*/
   /*#--------------------------------------------------*/

   bl inst_addr_break_handler
   b  finish_exception

   /*#-------------------------------------*/
   /*# System Management Exception - 01400*/
   /*#-------------------------------------*/

   bl sys_management_handler
   b  finish_exception

/* These handlers are not implemented */
	

machine_check_handler:
dsi_handler:
isi_handler:
external_handler:
alignment_handler:
program_handler:
float_unavail_handler:
system_call_handler:
trace_handler:
inst_trans_miss_handler:
data_load_miss_handler:
data_store_miss_handler:
inst_addr_break_handler:
sys_management_handler:

unknown_exception:
	bl	fatal_error_exception
	b	finish_exception


